Package: kollaR
Type: Package
Title: Event Classification, Visualization and Analysis of Eye Tracking
        Data
Version: 1.1.4
Authors@R: person(given = c("Johan", "Lundin"),
                  family = "Kleberg",
                  role = c("aut", "cre"),
                  email = "johan.lundin.kleberg@su.se")
Description: Functions for analysing eye tracking data, including event detection, visualizations and area of interest (AOI) based analyses. 
  The package includes implementations of the IV-T, I-DT, adaptive velocity threshold, and Identification by two means clustering (I2MC) algorithms.
  See separate documentation for each function. The principles underlying I-VT and I-DT algorithms are described in Salvucci & Goldberg (2000) <doi:10.1145/355017.355028>. 
  Two-means clustering is described in Hessels et al. (2017), <doi: 10.3758/s13428-016-0822-1>. 
  The adaptive velocity threshold algorithm is described in Nyström & Holmqvist (2010),<doi:10.3758/BRM.42.1.188>. 
  A documentation of the 'kollaR' can be found in Kleberg et al (2026) <doi:10.3758/s13428-025-02903-z>. Cite this paper when using 'kollaR'
  See a demonstration in the URL.
URL: https://drjohanlk.github.io/kollaR/demo.html
License: GPL-3
Encoding: UTF-8
LazyData: true
Imports: dplyr, ggplot2, zoo, ggforce, tidyr, ggpubr, jpeg, patchwork,
        shiny, base64enc, magick, scales
Config/roxygen2/version: 8.0.0
NeedsCompilation: no
Packaged: 2026-05-08 12:48:21 UTC; jolu7095
Author: Johan Lundin Kleberg [aut, cre]
Maintainer: Johan Lundin Kleberg <johan.lundin.kleberg@su.se>
Depends: R (>= 3.5.0)
Repository: CRAN
Date/Publication: 2026-05-08 13:10:02 UTC
Built: R 4.5.2; ; 2026-05-08 15:48:53 UTC; unix
