import(zoo)
import(dplyr)
import(ggplot2)
import(ggforce)
import(scales)
import(tidyr)
import(ggpubr)
import(shiny)
import(base64enc)
import(jpeg)
import(patchwork)
import(magick)
importFrom("stats", "kmeans", "median", "offset", "sd", "setNames")
importFrom("utils", "tail")
importFrom("stats", "mad")
exportPattern("^[[:alpha:]]+")
