% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/PreprocessingFunctions.R
\name{movmean.filter}
\alias{movmean.filter}
\title{Calculate the moving mean of a vector}
\usage{
movmean.filter(x, k = 3)
}
\arguments{
\item{x}{data to process}

\item{k}{window length}
}
\value{
filtered data
}
\description{
Calculate the moving mean of a vector while ignoring NAs. Used internally in pre-processing functions
}
